#include "aiplayer.h"

#include "GameMaster.h"

AIThing::AIThing(AIPlayer* player)
	: PegThing()
	, m_pPlayer(player)
{
}
	
/////////////////////////////////////////////////////////////////////////////

void AIThing::wakeMeUpLater()
{
	SetTimer(WAITING_FOR_PLAYING, TIME_BEFORE_AI_MOVE,0);
}
	
/////////////////////////////////////////////////////////////////////////////

void AIThing::boatsPlaced()
{
	PegMessage msg(this, BOATS_PLACED);
	MessageQueue()->Push(msg);	
}

/////////////////////////////////////////////////////////////////////////////

SIGNED AIThing::Message(const PegMessage &msg)
{
	switch(msg.wType)
	{
	case BOATS_PLACED:
		m_pPlayer->readyToStart();
		break;

	case PM_TIMER:
		if(msg.iData==WAITING_FOR_PLAYING) m_pPlayer->timeToRox();
		break;

	default:
		return PegThing::Message(msg);
	}

	return 0;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

AIPlayer::AIPlayer(int num, GameMaster* master)
	: Player(num,master)
{
	m_thing = new AIThing(this);
	m_trackedBoat = -1;
}

/////////////////////////////////////////////////////////////////////////////

AIPlayer::~AIPlayer(void)
{
	delete m_thing;
}

/////////////////////////////////////////////////////////////////////////////

void AIPlayer::yourTurn()
{
	m_thing->wakeMeUpLater();
}

/////////////////////////////////////////////////////////////////////////////

void AIPlayer::placeYourBoats()
{
	placeBoatsAuto();
	m_thing->boatsPlaced();
}

/////////////////////////////////////////////////////////////////////////////

void AIPlayer::timeToRox()
{
	int row, col;

	if(m_trackedBoat!=-1)
	{
		Player* opp = master()->opponent(number());
		AIBoatData &data = m_knowings[m_trackedBoat];

		if(data.direc==UNKNOWN)	// we just found one case of this boat
		{
			bool top = (data.row==0 || opp->getKnownCaseType(data.row-1,data.col)!=UnKnown);
			bool bottom = (data.row==SIZE_GRID-1 || opp->getKnownCaseType(data.row+1,data.col)!=UnKnown);
			bool left = (data.col==0 || opp->getKnownCaseType(data.row,data.col-1)!=UnKnown);
			bool right = (data.col==SIZE_GRID-1 || opp->getKnownCaseType(data.row,data.col+1)!=UnKnown);

			if(top && bottom) data.direc = HORIZ;
			else if(left && right) data.direc = VERTIC;
			else	// if we still don't know the direction of this boat
			{
				row = data.row;
				col = data.col;
				if(top) row++;	// try at the bottom
				else if(bottom) row--;	// try at the top
				else if(left) col++; // try on the right
				else if(right) col--; // try on the left
				else row--;	// try at the top
			}
		}
		
		if(data.direc!=UNKNOWN)	// if we found this direction
		{
			if(data.direc==VERTIC)
			{
				col = data.col;

				if(data.row==0 || opp->getKnownCaseType(data.row-1,data.col)!=UnKnown)
				{
					row = -1;
					for(int i=1; row==-1 && i<data.boat->size; i++)
						if(opp->getKnownCaseType(data.row+i,data.col)==UnKnown) 
							row = data.row+i;

					ASSERT(row!=-1);
				}
				else row = data.row-1;
			}
			else
			{
				row = data.row;

				if(data.col==0 || opp->getKnownCaseType(data.row,data.col-1)!=UnKnown)
				{
					col = -1;
					for(int i=1; col==-1 && i<data.boat->size; i++)
						if(opp->getKnownCaseType(data.row,data.col+i)==UnKnown) 
							col = data.col+i;

					ASSERT(col!=-1);
				}
				else col = data.col-1;
			}
		}
	}
	else
	{		
		do
		{
			col = random(SIZE_GRID);
			row = random((SIZE_GRID + !(col%2))/2)*2 + col%2;
		}
		while(master()->opponent(number())->getKnownCaseType(row,col) != UnKnown);
	}

	shoot(row,col);
}

/////////////////////////////////////////////////////////////////////////////

void AIPlayer::readyToStart()
{
	master()->boatsPlaced(number());
}

/////////////////////////////////////////////////////////////////////////////

void AIPlayer::myShotResult(const ShotResult &res)
{
	if(m_trackedBoat==-1 && res.boat) m_trackedBoat = res.boat->id;
	analyzeShot(res);
}

/////////////////////////////////////////////////////////////////////////////

void AIPlayer::analyzeShot(const ShotResult &res)
{
	if(res.boat==NULL) return;

	AIBoatData &data = m_knowings[res.boat->id];

	if(data.boat != NULL)	// if we already found that boat
	{
		if(res.boat->destroyed)
		{
			if(m_trackedBoat==res.boat->id)
			{
				m_trackedBoat = -1;
				for(int i=0; i<NBR_BOATS; i++)
					if(m_knowings[i].boat && m_knowings[i].boat->destroyed==false)
						{ m_trackedBoat = i; break; }
			}
		}
		else
		{
			if(data.direc==UNKNOWN) data.direc = (data.col==res.col ? VERTIC : HORIZ);
			if(data.direc==VERTIC) { if(res.row < data.row) data.row = res.row; }
			else { if(res.col < data.col) data.col = res.col; }
		}
	}
	else
	{
		data.boat = res.boat;
		data.col = res.col;
		data.row = res.row;
	}
}

/////////////////////////////////////////////////////////////////////////////

void AIPlayer::opponentShotResult(const ShotResult &res)
{

}

/////////////////////////////////////////////////////////////////////////////


