#include "../USBComm/USBComm.h"
#include "../MainFrame/MyMainFrame.h"
#include "DistantPlayer.h"
#include "OptionsWdw.h"
#include <ClassPad.h>

#define RECEIVE(buff, size, bool) bool = (bool && linkReceive((byte*)buff, size)==0)
#define SEND(buff, size, bool) bool = (bool && linkSend((byte*)buff, size)==0)

#define TEST_RECEPTION			10

#define HELLO_IM_HERE			35
#define PLAYER_IS_READY			17
#define IM_READY_TOO			83
#define SHOT_RESULT				72
#define SHOT_COORD				49
#define GAME_STOPPED			57
#define TIMEOUT					38

#define SHOT_DONE				1
#define WAITING_SHOT			2
#define WAITING_PLAYER_READY	3

#ifdef WIN32
#define TEST_FREQUENCY			20
#else
#define TEST_FREQUENCY			6
#endif

#define TIME_OUT				100

DistantThing::DistantThing(DistantPlayer* p)
{
	m_player = p;
}

/////////////////////////////////////////////////////////////////////////////

DistantThing::~DistantThing()
{
	KillTimer(TEST_RECEPTION);
}

/////////////////////////////////////////////////////////////////////////////

void DistantThing::waitMessage(char* text)
{
	MyMainFrame::mf()->statusBarText(text);
	m_cpt = 0;
	SetTimer(TEST_RECEPTION, TEST_FREQUENCY, 1);
	testConnection();
}

/////////////////////////////////////////////////////////////////////////////

SIGNED DistantThing::Message(const PegMessage &msg)
{
	if(msg.wType==PM_TIMER && msg.iData==TEST_RECEPTION)
	{
		testConnection();
		return 0;
	}
	else return PegThing::Message(msg);
}

/////////////////////////////////////////////////////////////////////////////

void DistantThing::testConnection()
{
	byte b=0;
	m_cpt++;
	word res = linkReceive(b);
	if(res==0 || m_cpt==TIME_OUT) 
	{
		KillTimer(TEST_RECEPTION);
		m_player->newMessage( res==0 ? b : TIMEOUT );
		m_cpt=0;
	}
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

DistantPlayer::DistantPlayer(int num, GameMaster* master)
	: Player(num, master)
	, m_thing(this)
	, m_boat(0,BOAT_BS)
	, m_state(0)
{
	m_imReady = m_otherReady = false;

	MyMainFrame::mf()->statusBarText(GetLang(MSG_WAIT_CALL));

	if(linkOpen(IM_LINK_115200 | (OptionsWdw::linkUsbMode() ? IM_LINK_USB : IM_LINK_SRL3PIN) ))
	{
		PegMessageWindow* pWin = new PegMessageWindow(GetLang(CAPT_CONNECTION), GetLang(MSG_OPEN_PORT_FAILED), MW_OK);
		pWin->Execute();
		m_initOk = false;
		MyMainFrame::mf()->statusBarText("");
		return;
	}

	short prior = random(20000);
	byte msg[10] = { HELLO_IM_HERE, (byte)(prior/256), (byte)(prior%256) };
	linkSend(msg,3);

	msg[0] = 0;
	word res = linkReceive(msg,3);
	
	if(res==0 && msg[0]==HELLO_IM_HERE)
	{
		short other = ((short)msg[1])*256 + msg[2];
		m_priority = (prior>other ? 0 : 1);
		m_initOk = true;
		//PegMessageWindow* pWin = new PegMessageWindow(GetLang(CAPT_CONNECTION), m_priority? "I'm server" : "I'm client",MW_OK);
		//pWin->Execute();
	}
	else 
	{
		PegMessageWindow* pWin = new PegMessageWindow(GetLang(CAPT_CONNECTION), GetLang(MSG_NO_RESPONSE),MW_OK);
		pWin->Execute();
		m_initOk = false;
	}

	MyMainFrame::mf()->statusBarText("");
}

/////////////////////////////////////////////////////////////////////////////

DistantPlayer::~DistantPlayer()
{
	linkSend(GAME_STOPPED);
	linkClose(IMW_COMM_NORMAL);
}

/////////////////////////////////////////////////////////////////////////////

void DistantPlayer::newMessage(byte msg)
{
	bool ok;
	word w = 23;

	switch(msg)
	{
	case SHOT_COORD:
	{
		ok = (m_state==WAITING_SHOT);
		
		byte coord[2];
		ok = ok && (w = linkReceive(coord,2))==0;

		if(ok) 
		{
			m_state = SHOT_DONE;
			shoot(coord[0], coord[1]);
		}
		break;
	}

	case PLAYER_IS_READY: case IM_READY_TOO:
	{
		ok = (m_state==WAITING_PLAYER_READY);

		if(ok) 
		{
			m_otherReady = true;
			if(m_imReady && msg==PLAYER_IS_READY) ok = (linkSend(IM_READY_TOO)==0);

			//PegMessageWindow* pWin = new PegMessageWindow(GetLang(CAPT_CONNECTION), "Player ready",MW_OK);
			//pWin->Execute();

			master()->boatsPlaced(number());
		}
		break;
	}

	case GAME_STOPPED: case TIMEOUT:
		ok = false;
		break;

	default:
		ok = false;
		break;
	}
			
	if(!ok) lostConnect();
}

/////////////////////////////////////////////////////////////////////////////

void DistantPlayer::lostConnect()
{
	PegMessageWindow* pWin = new PegMessageWindow(GetLang(CAPT_ERROR), GetLang(MSG_CONNECT_ERROR), MW_OK);
	pWin->Execute();
	master()->stopGame();
}

/////////////////////////////////////////////////////////////////////////////

void DistantPlayer::yourTurn()
{
	m_state = WAITING_SHOT;
	// waiting shot coordinates
	m_thing.waitMessage(GetLang(MSG_WAIT_OPP_SHOT));
}

/////////////////////////////////////////////////////////////////////////////

void DistantPlayer::opponentIsReady()
{
	// LINK : send message 'PLAYER_IS_READY'
	linkSend(PLAYER_IS_READY);

	m_state = WAITING_PLAYER_READY;
	m_thing.waitMessage(GetLang(MSG_WAIT_OPP_READY));
}

/////////////////////////////////////////////////////////////////////////////

void DistantPlayer::placeYourBoats()
{
}

/////////////////////////////////////////////////////////////////////////////

const ShotResult & DistantPlayer::opponentShot(int row, int col)
{
	bool ok=true;
	
	// LINK : send shot coordinates
	byte coord[2] = { (byte)row, (byte)col };
	ok = (linkSend(SHOT_COORD)==0);
	ok = (ok && linkSend(coord, 2)==0);

	// LINK : immediatly wait for result
	if(ok)
	{
		//PegMessageWindow* pWin = new PegMessageWindow(GetLang(CAPT_CONNECTION), "Waiting Result",MW_OK);
		//PegThing::Presentation()->Add(pWin);

		byte msg;
		int i;
		for(i=0; i<100 && (linkReceive(msg)!=0); i++) ;
		if(i==100 || msg!=SHOT_RESULT) ok = false;
		
		ok = ok && receiveResult();
		if(ok) grid()->acceptsResult(m_lastRes);
	}
		
	if(!ok) lostConnect();

	return m_lastRes;
}

/////////////////////////////////////////////////////////////////////////////

bool DistantPlayer::sendResult(const ShotResult &res)
{
	byte data[] = { res.row, res.col, res.win, (res.boat!=NULL ? 1 : 0) };
	bool ok = linkSend(data, sizeof(data))==0;

	if(ok && res.boat)
	{
		Boat* b = res.boat;
		byte data2[] = { b->damage, b->destroyed, b->hit, b->id, b->size, b->type };
		ok = linkSend(data2,sizeof(data2))==0;
	} 
	
	return ok;
}

/////////////////////////////////////////////////////////////////////////////

bool DistantPlayer::receiveResult()
{
	bool ok = true;
	ShotResult &res = m_lastRes;

	byte data[6];
	ok = linkReceive(data, 4)==0;
	if(ok)
	{
        res.row = data[0];
        res.col = data[1];
        res.win = data[2];

		if(data[3]==1)
		{
			Boat* b = &m_boat;
			ok = linkReceive(data, 6)==0;
			if(ok)
			{
				b->damage = data[0];
				b->destroyed = data[1];
				b->hit = data[2];
				b->id = data[3];
				b->size = data[4];
				b->type = data[5];
                res.boat = b;
			}
		} 
		else res.boat = NULL;
	}
	
	return ok;
}

/////////////////////////////////////////////////////////////////////////////

void DistantPlayer::myShotResult(const ShotResult &res)
{
		//PegMessageWindow* pWin = new PegMessageWindow(GetLang(CAPT_CONNECTION), "Sending Result",MW_OK);
		//pWin->Execute();

	// LINK : immediatly send result
	bool ok = (linkSend(SHOT_RESULT)==0);
	ok = ok && sendResult(res);

	if(!ok) lostConnect();
}

/////////////////////////////////////////////////////////////////////////////

