#include "GameMaster.h"
#include "LocalPlayer.h"
#include "AIPlayer.h"
#include "DistantPlayer.h"
#include "IntroWdw.h"

GameMaster::GameMaster(byte gameType, IntroWdw* intro)
	: PegThing()
	, m_intro(intro)
{
	m_pPlayers[0] = new LocalPlayer(this);
	if(gameType==ONE_PLAYER) 	m_pPlayers[1] = new AIPlayer(1, this);
	else 						m_pPlayers[1] = new DistantPlayer(1,this);

	m_initOk = m_pPlayers[0]->initOk() && m_pPlayers[1]->initOk();

	if(m_initOk)
	{
        m_p0Ready = m_p1Ready = false;
		m_pPlayers[0]->placeYourBoats();
		m_pPlayers[1]->placeYourBoats();
	}
}

/////////////////////////////////////////////////////////////////////////////

GameMaster::~GameMaster()
{
	delete m_pPlayers[0];
	delete m_pPlayers[1];
}

/////////////////////////////////////////////////////////////////////////////

PlayerDoc* GameMaster::getLocalPlayerDoc() 
{ 
	return (LocalPlayer*) m_pPlayers[0]; 
}

/////////////////////////////////////////////////////////////////////////////

const ShotResult &  GameMaster::playerShot(int player, int row, int col)
{
	const ShotResult &res = m_pPlayers[!player]->opponentShot(row,col);

	if(res.win) setNewMessage( player? PLAYER1_WON : PLAYER0_WON );
	else setNewMessage( player? PLAYER0_TURN : PLAYER1_TURN );

	return res;
}

/////////////////////////////////////////////////////////////////////////////

void GameMaster::boatsPlaced(int num)
{
	if(num) m_p1Ready = true; 
	else m_p0Ready = true;
	m_pPlayers[!num]->opponentIsReady();

	if(m_p1Ready && m_p0Ready)
	{
		//PegMessageWindow* pWin = new PegMessageWindow("Test", "Launching game",MW_OK);
		//pWin->Execute();

		setNewMessage( m_pPlayers[0]->priority() > m_pPlayers[1]->priority() ? PLAYER0_TURN : PLAYER1_TURN );
	}
}

/////////////////////////////////////////////////////////////////////////////

void GameMaster::stopGame()
{
	m_intro->gameEnded();
}

/////////////////////////////////////////////////////////////////////////////

SIGNED GameMaster::Message(const PegMessage &msg)
{
	PegRect r = {21,100,138,150};
	PegMessageWindow* wind;

	switch(msg.wType)
	{
	case PLAYER0_TURN: 
		m_pPlayers[0]->yourTurn();
		break;

	case PLAYER1_TURN:
		m_pPlayers[1]->yourTurn();
		break;

	case PLAYER0_WON:
		wind = new PegMessageWindow(r, GetLang(CAPT_END_GAME), GetLang(MSGBOX_WIN));
		wind->Execute();
		m_intro->gameEnded();
		break;

	case PLAYER1_WON:
		wind = new PegMessageWindow(r, GetLang(CAPT_END_GAME), GetLang(MSGBOX_LOSE));
		wind->Execute();
		m_intro->gameEnded();
		break;

	default: 
		return PegThing::Message(msg);
	}

	return 0;

}

/////////////////////////////////////////////////////////////////////////////

