#include "griddata.h"

GridData::GridData()
{
	for(int i=0; i<NBR_BOATS; i++) m_boats[i]=NULL;
	clear();
}

/////////////////////////////////////////////////////////////////////////////

GridData::~GridData()
{
	clear();
}

/////////////////////////////////////////////////////////////////////////////

void GridData::acceptsResult(const ShotResult &res)
{
	if(res.boat)
	{
		int id = res.boat->id;
		if(m_boats[id]==NULL) m_boats[id] = new Boat(id, res.boat->type);
		*(m_boats[id]) = *(res.boat);
		m_grid[res.row][res.col].setBoat(m_boats[id]);
		m_grid[res.row][res.col].open();
		if(res.boat->destroyed) m_nbrDestroyedBoats++;
	}
	else m_grid[res.row][res.col].open();
}

/////////////////////////////////////////////////////////////////////////////

const ShotResult & GridData::opponentShot(int row, int col)
{
	m_lastRes.row = row;
	m_lastRes.col = col;
	if(m_grid[row][col].hit(m_lastRes))
	{
		if((++m_nbrDestroyedBoats) == NBR_BOATS)
		{
			CaseData::finished = true;
			m_lastRes.win = true;
		}
		else m_lastRes.win = false;
	}
	return m_lastRes;
}

/////////////////////////////////////////////////////////////////////////////

bool GridData::canPlaceNewBoat(int row, int col, bool horiz, int size)
{
	if(size<boatSizes[BOAT_MS] || size>boatSizes[BOAT_BS]) return false;
	
	switch(size)
	{
	case 5: 
		if(m_nbrPlacedBoats[BOAT_BS]==NBR_BOAT_BS) return false;
		break;
	case 4: 
		if(m_nbrPlacedBoats[BOAT_FR]==NBR_BOAT_FR) return false;
		break;
	case 3: 
		if(m_nbrPlacedBoats[BOAT_CR]==NBR_BOAT_CR) return false;
		break;
	case 2: 
		if(m_nbrPlacedBoats[BOAT_MS]==NBR_BOAT_MS) return false;
		break;
	}

	bool ok;
	if(horiz)
	{
		ok = (row < SIZE_GRID && row >= 0 && col >= 0);
		for(int i=0; ok && i<size; i++) 
			ok = (col+i < SIZE_GRID && m_grid[row][col+i].hasBoat()==false);
	}
	else
	{
		ok = (col < SIZE_GRID && col >= 0 && row >= 0);
		for(int i=0; ok && i<size; i++) 
			ok = (row+i < SIZE_GRID && m_grid[row+i][col].hasBoat()==false);
	}

	return ok;
}

/////////////////////////////////////////////////////////////////////////////

void GridData::placeBoat(int row, int col, bool horiz, Boat* b)
{
	int size = b->size;
	
	if(horiz)
	{
		for(int i=0; i<size; i++) m_grid[row][col+i].setBoat(b);
	}
	else
	{
		for(int i=0; i<size; i++) m_grid[row+i][col].setBoat(b);
	}

	b->row = row;
	b->col = col;
	b->horiz = horiz;
	m_boats[b->id] = b;
	m_nbrPlacedBoats[b->type]++;
}

/////////////////////////////////////////////////////////////////////////////

int GridData::getNewIdForSize(int size)
{
	int base=0;
	if(size<boatSizes[BOAT_BS]) base += NBR_BOAT_BS;
	if(size<boatSizes[BOAT_FR]) base += NBR_BOAT_FR;
	if(size<boatSizes[BOAT_CR]) base += NBR_BOAT_CR;

	while(m_boats[base]!=NULL) base++;
	return base;
}

/////////////////////////////////////////////////////////////////////////////

bool GridData::allBoatsPlaced()
{
	bool ok=true;
	for(int i=0; ok && i<NBR_BOATS; i++) ok = (m_boats[i]!=NULL);
	return ok;
}

/////////////////////////////////////////////////////////////////////////////

void GridData::removeBoat(int id)
{
	int size = m_boats[id]->size;
	int row = m_boats[id]->row;
	int col = m_boats[id]->col;

	if(m_boats[id]->horiz)
	{
		for(int i=0; i<size; i++) m_grid[row][col+i].setBoat(NULL);
	}
	else
	{
		for(int i=0; i<size; i++) m_grid[row+i][col].setBoat(NULL);
	}

	m_nbrPlacedBoats[ m_boats[id]->type ]--;
	delete m_boats[id];
	m_boats[id] = NULL;
}

/////////////////////////////////////////////////////////////////////////////

void GridData::clear()
{
	int i;
	for(i=0; i<4; i++) m_nbrPlacedBoats[i] = 0;
	for(i=0; i<SIZE_GRID; i++) for(int j=0; j<SIZE_GRID; j++) 
		m_grid[i][j].setBoat(NULL);
	m_nbrDestroyedBoats = 0;

	for(i=0; i<NBR_BOATS; i++) 
		if(m_boats[i]) { delete m_boats[i]; m_boats[i]=NULL; }

	CaseData::finished = false;
}

/////////////////////////////////////////////////////////////////////////////

