#include "IntroWdw.h"
#include "GameMaster.h"
#include "PlayerWdw.h"
#include "OptionsWdw.h"
#include "../MainFrame/MyMainFrame.h" 

#define TIMER_START		567

#define BTN_NEWGAME		1500
#define BTN_OPTIONS		1501
#define BTN_ONE_PLAYER	1502
#define BTN_TWO_PLAYERS	1503

PegRect IntroWdw::m_rect = { 14,53,143,153 };

PegMenuDescriptionML IntroWdw::m_menuDesc[] = {
	{NULL,	MENU_OPTIONS,	BTN_OPTIONS,	AF_ENABLED,		NULL},
	{NULL,	MENU_NEWGAME,	BTN_NEWGAME,	AF_ENABLED,		NULL},
	{ "",	CMN_NO_ID,		0,				0,				NULL} };

/////////////////////////////////////////////////////////////////////////////

IntroWdw::IntroWdw()
	: MyWindow(m_rect,FF_THIN)
{
	m_window = NULL;
	m_gameRunning = false;
	mainFrame()->getClientArea()->SetScrollMode(WSM_AUTOVSCROLL);
	mainFrame()->setMenuBar( new PegMenuBar(m_menuDesc) );
	declareSignal( SIGNAL(BTN_NEWGAME,PSF_CLICKED) );
	declareSignal( SIGNAL(BTN_OPTIONS,PSF_CLICKED) );

	AddR( new PegTextButton(18, 42, 92, GetLang(BTNTXT_ONE_PLAY), BTN_ONE_PLAYER, AF_ENABLED|TJ_CENTER) );
	AddR( new PegTextButton(18, 63, 92, GetLang(BTNTXT_TWO_PLAY), BTN_TWO_PLAYERS, AF_ENABLED|TJ_CENTER) );
	declareSignal( SIGNAL(BTN_ONE_PLAYER,PSF_CLICKED) );
	declareSignal( SIGNAL(BTN_TWO_PLAYERS,PSF_CLICKED) );

	OptionsWdw::enable();
	OptionsWdw::loadOptions();
}

/////////////////////////////////////////////////////////////////////////////

IntroWdw::~IntroWdw(void)
{
}

/////////////////////////////////////////////////////////////////////////////

void IntroWdw::onDraw()
{
	PegPoint p = { 17, 8 };
	Window2Screen(p);
	Bitmap(p,&bmpTitle,true);

	DrawChildren();

	PegColor col(BLACK);
	p.x = 3;
	p.y = 89;
	DrawTextR(p,"Patte J.",col,PegTextThing::GetDefaultFont(0));
	p.x = 102;
	DrawTextR(p,"2005",col,PegTextThing::GetDefaultFont(0));
}

/////////////////////////////////////////////////////////////////////////////

void IntroWdw::onSignal(WORD signal)
{
	switch(signal)
	{
	case SIGNAL(BTN_NEWGAME,PSF_CLICKED):		newGame();					break;
	case SIGNAL(BTN_OPTIONS,PSF_CLICKED):		OptionsWdw::showWindow();	break;
	case SIGNAL(BTN_ONE_PLAYER,PSF_CLICKED):	startNewGame(ONE_PLAYER);	break;
	case SIGNAL(BTN_TWO_PLAYERS,PSF_CLICKED):	startNewGame(TWO_PLAYERS);	break;
	default: ;
	}
}

/////////////////////////////////////////////////////////////////////////////

void IntroWdw::newGame()
{
	if(m_gameRunning)
	{
		PegMessageWindow* pWin = new PegMessageWindow(GetLang(CAPT_NEW_GAME), GetLang(MSG_ASK_CONFIRM), MW_YES|MW_NO);
		if( pWin->Execute() !=IDB_YES ) return;
	}
	
	clearGame();
}

/////////////////////////////////////////////////////////////////////////////

void IntroWdw::clearGame()
{
	if(m_window==NULL) return;
	
	mainFrame()->removeWindow(m_window, true);
	m_window = NULL;
	delete m_master;
	m_gameRunning = false;
	mainFrame()->statusBarText("");
	OptionsWdw::enable();
}

/////////////////////////////////////////////////////////////////////////////

void IntroWdw::startNewGame(byte type)
{
	m_master = new GameMaster(type, this);

	if(m_master->initOk())
	{
		m_window = new PlayerWdw(m_master->getLocalPlayerDoc());
		mainFrame()->addWindow(m_window);
		m_gameRunning = true;
	}
	else delete m_master;
}

/////////////////////////////////////////////////////////////////////////////
