#include "localplayer.h"
#include "OptionsWdw.h"
#include "../MainFrame/MyMainFrame.h"

LocalPlayer::LocalPlayer(GameMaster* master)
	: Player(0,master)
	//, PlayerDoc(mf)
{
	m_myTurn = false;
	m_placingBoats = false;
	for(int i=0; i<NBR_BOATS; i++)
	{
		m_damages[0][i] = -1;
		m_damages[1][i] = 0;
	}
}

/////////////////////////////////////////////////////////////////////////////

void LocalPlayer::yourTurn()
{
	OptionsWdw::enable();
	m_myTurn = true;
}

/////////////////////////////////////////////////////////////////////////////

void LocalPlayer::placeYourBoats()
{
	if(OptionsWdw::placeBoatsManually())
	{
		MyMainFrame::mf()->statusBarText(GetLang(MSG_PLACE_BOATS));
		m_placingBoats = true;
	}
	else
	{
		placeBoatsAuto();
		for(int i=0; i<NBR_BOATS; i++) m_damages[0][i] = 0;
		master()->boatsPlaced(number());
	}
}

/////////////////////////////////////////////////////////////////////////////

bool LocalPlayer::canPlaceBoat(int row, int col, bool horiz, int size)
{
	return grid()->canPlaceNewBoat(row,col,horiz,size);
}

/////////////////////////////////////////////////////////////////////////////

void LocalPlayer::placeBoat(int row, int col, bool horiz, int size)
{
	int id = grid()->getNewIdForSize(size);
	byte types[] = { BOAT_MS, BOAT_CR, BOAT_FR, BOAT_BS };

	grid()->placeBoat(row,col,horiz, new Boat(id, types[size-2]) );
	m_damages[0][id] = 0;

	if(grid()->allBoatsPlaced()) 
	{
		m_placingBoats = false;
		master()->boatsPlaced(number());
	}
}

/////////////////////////////////////////////////////////////////////////////

void LocalPlayer::myShotResult(const ShotResult &res)
{
	UpdateOrderType order;
	if(res.boat==NULL) order = MeMissed;
	else
	{
		if(res.win) order = PlayerWon;
		else if(res.boat->destroyed) order = MeDestroyed;
		else order = MeHit;
		m_damages[1][res.boat->id] = res.boat->damage;
	}

	m_rowChanged = res.row;
	m_colChanged = res.col;
	m_window->newOrder(order);
}

/////////////////////////////////////////////////////////////////////////////

void LocalPlayer::opponentShotResult(const ShotResult &res)
{
	UpdateOrderType order;
	if(res.boat==NULL) order = OpponentMissed;
	else
	{
		if(res.win) order = PlayerLost;
		else if(res.boat->destroyed) order = OpponentDestroyed;
		else order = OpponentHit;
		m_damages[0][res.boat->id] = res.boat->damage;
	}

	m_rowChanged = res.row;
	m_colChanged = res.col;
	m_window->newOrder(order);
}

/////////////////////////////////////////////////////////////////////////////

void LocalPlayer::redrawWnd()
{
	m_window->newOrder(ReDraw);
}

/////////////////////////////////////////////////////////////////////////////

void LocalPlayer::shoot(int row, int col)
{
	OptionsWdw::enable(false);
	m_myTurn = false;
	Player::shoot(row,col);
}

/////////////////////////////////////////////////////////////////////////////

CaseType LocalPlayer::getCaseType(int player, int row, int col)
{
	return ( player==number() ? getRealCaseType(row,col)
				: master()->opponent(number())->getKnownCaseType(row,col) ); 
}

/////////////////////////////////////////////////////////////////////////////

void LocalPlayer::getChangedCase(int &row, int &col)
{
	row = m_rowChanged;
	col = m_colChanged;
}

/////////////////////////////////////////////////////////////////////////////


