#include "PlayerWdw.h"
#include "Player.h"
#include "../MainFrame/MyMainFrame.h" 
#include "OptionsWdw.h"

PegRect PlayerWdw::m_rect = { 0,0,159,210 };

/////////////////////////////////////////////////////////////////////////////
//	Constructor
/////
//
PlayerWdw::PlayerWdw(PlayerDoc* doc)
	: MyWindow(m_rect, FF_NONE)
	, m_doc(doc)
{
	doc->setWindow(this);
}

/////////////////////////////////////////////////////////////////////////////

void PlayerWdw::sayStatusBar(PEGCHAR* s) 
{ 
	mainFrame()->statusBarText(s); 
}

/////////////////////////////////////////////////////////////////////////////

void PlayerWdw::drawFrame(int player, int x, int y)
{
	PegColor black(BLACK), white(WHITE);
	PegRect r = { x, y, x+FRAME_WIDTH, y+FRAME_HEIGHT };
	RectangleR(r,black);

	PegFont* font = PegTextThing::GetDefaultFont(0);
	char* text = GetLang( player==0 ? TITLE_PLAYER : TITLE_OPPONENT );
	int width = Screen()->TextWidth(text,font) + 10;

	if(player==0)
	{
		int right = x+width;
		r.Set(x, y-11, right, y);
		RectangleR(r, black);
		LineR(x+1, y, right-1, y, white);
		PegPoint p = { x+6, y-9 };
		DrawTextR(p, text, black, font);
	}
	else
	{
		int left = x+FRAME_WIDTH-width;
		r.Set(left, y-11, x+FRAME_WIDTH, y);
		RectangleR(r, black);
		LineR(left+1, y, x+FRAME_WIDTH-1, y, white);
		PegPoint p = { left+6, y-9 };
		DrawTextR(p, text, black, font);
	}

}

/////////////////////////////////////////////////////////////////////////////

void PlayerWdw::drawGrid(int player, int x, int y)
{
	PegPoint pos = {x, y};
	Window2Screen(pos);
	Bitmap(pos,&bmpGrid,true);
	
	for(int i=0; i<SIZE_GRID; i++) for(int j=0; j<SIZE_GRID; j++)
	{
		PegPoint point = { (x+1)+(SIZE_CASE+1)*j, (y+1)+(SIZE_CASE+1)*i };
		Window2Screen(point);
		Bitmap(point, caseBitmaps[ playerDoc()->getCaseType(player,i,j) ], true);
	}
}

/////////////////////////////////////////////////////////////////////////////

void PlayerWdw::drawBoatStates(int player, int x, int y)
{
	PegPoint framePt = {x,y}, pos = {x+4,y+3}, textPt = { x + 34, y+2 };
	Window2Screen(framePt);
	Window2Screen(pos);
	Window2Screen(textPt);

	char s[] = "0/0";	
	PegColor col(BLACK);

	Bitmap(framePt, &bmpBoatFrame, true);

	for(int i=0; i<NBR_BOATS; i++)
	{
		byte type = boatTypes[i];
		Bitmap(pos, boatBitmaps[type], true);
		
		int dmg = playerDoc()->boatDamage(player, i);
		if(dmg!=-1)
		{
			s[0] = '0' + dmg;
			s[2] = '0' + boatSizes[type];
			DrawText(textPt, s, col, PegTextThing::GetBasicFont());
		}

		pos.y += BOATS_HSPACE;
		textPt.y += BOATS_HSPACE;
	}
}

/////////////////////////////////////////////////////////////////////////////

void PlayerWdw::drawPlacingBoat()
{
	int r = m_boatTop;
	int c = m_boatLeft;
	for(int i=0; i<m_boatSize; i++)
	{
		PegPoint point = 
			{ (ME_GRID_POS_X+1)+(SIZE_CASE+1)*c, (ME_GRID_POS_Y+1)+(SIZE_CASE+1)*r };
		Window2Screen(point);
		Bitmap(point, caseBitmaps[ m_boatValid ? Valid : UnValid ], true);
		if(m_boatHoriz) c++;
		else r++;
	}
}

/////////////////////////////////////////////////////////////////////////////
//	Global update of the screen
/////
//
void PlayerWdw::onDraw()
{
		// Player's data
	drawFrame(0,ME_GRID_POS_X-3,ME_GRID_POS_Y-3);
	drawGrid(0, ME_GRID_POS_X, ME_GRID_POS_Y);
	drawBoatStates(0, ME_BOATS_POS_X, ME_BOATS_POS_Y);
	if(playerDoc()->placingBoats() && m_boatBasePlaced) drawPlacingBoat();

	// Opponent's data
	drawFrame(1,OPP_BOATS_POS_X-3,OPP_BOATS_POS_Y-3);
	drawGrid(1, OPP_GRID_POS_X, OPP_GRID_POS_Y);
	drawBoatStates(1, OPP_BOATS_POS_X, OPP_BOATS_POS_Y);
}

/////////////////////////////////////////////////////////////////////////////
//	Return the topleft pixel corresponding to a certain case
/////
//
PegPoint PlayerWdw::getPxlForCase(int player, int row, int col)
{
	PegPoint p;
	p.x = (player==0?ME_GRID_POS_X:OPP_GRID_POS_X) + 4 + (SIZE_CASE+1)*col;
	p.y = (player==0?ME_GRID_POS_Y:OPP_GRID_POS_Y) + 4 + (SIZE_CASE+1)*row;
	return p;
}

/////////////////////////////////////////////////////////////////////////////
//	Utility function, currently missing in VisualStudio library
/////
//
#ifdef WIN32
extern "C" int Bevt_Wait(long time)
{
   DWORD start = GetTickCount();
   while (GetTickCount()-start < (DWORD)time) { }
   return TRUE;
}
#endif 

/////////////////////////////////////////////////////////////////////////////
//	Draw a small animation on screen by drawing some growing circles
/////
//
void PlayerWdw::drawExplosion(int num, PegPoint where)
{
	if(OptionsWdw::drawAnimations()==false)
	{
		Redraw();
		return;
	}
	
	// first update the screen
	Invalidate();
	Draw();

		// then capture the part of the screen covered by the animation
	Window2Screen(where);
	PegCapture capture;
	PegRect rect = { where.x-16, where.y-16, where.x+16, where.y+16 };
	Screen()->Capture(&capture, rect);

	PegColor col(BLACK,BLACK);

		// set circles' radius...
	int c1=0, c2=-30, c3=-30, max=15;
	if(num>=2) { c2 = -5; max+=5; }
	if(num==3) { c3 = -10; max+=5; }

		// now draw the animation
	for(int i=0; i<max; i++)
	{
		Invalidate();
		BeginDraw();

			// clear the previous circles
		Bitmap(capture.Point(), capture.Bitmap());
		
			// draw the new ones
		if(c1>0 && c1<15) Circle(where.x, where.y, c1, col, 3);
		if(c2>0 && c2<15) Circle(where.x, where.y, c2, col, 3);
		if(c3>0 && c3<15) Circle(where.x, where.y, c3, col, 3);
		c1++; c2++; c3++;
		
			// wait a moment...
		Bevt_Wait(EXPL_WAIT_TIME);

		EndDraw();
	}

		// finally, restore the updated screen
	Invalidate();
	BeginDraw();
	Bitmap(capture.Point(), capture.Bitmap());
	EndDraw();
}

/////////////////////////////////////////////////////////////////////////////
//	Business Logic Events Handler
/////
//
//void PlayerWdw::OnDataChanged()
void PlayerWdw::newOrder(UpdateOrderType order)
{
	int row, col;

	switch(order)
	{
	case MeMissed:
		sayStatusBar(MSG_ME_MISSED);
		drawExplosion(1, m_choosedPoint); 
		break;

	case MeHit:
		sayStatusBar(MSG_ME_HIT);
		drawExplosion(2, m_choosedPoint);  
		break;

	case MeDestroyed:
		sayStatusBar(MSG_ME_DESTROYED);
		drawExplosion(3, m_choosedPoint);  
		break;

	case OpponentMissed:
		sayStatusBar(MSG_OPP_MISSED);
		playerDoc()->getChangedCase(row, col);
		drawExplosion(1, getPxlForCase(0,row,col) );  
		break;

	case OpponentHit:
		sayStatusBar(MSG_OPP_HIT);
		playerDoc()->getChangedCase(row, col);
		drawExplosion(2, getPxlForCase(0,row,col) );  
		break;

	case OpponentDestroyed:
		sayStatusBar(MSG_OPP_DESTROYED);
		playerDoc()->getChangedCase(row, col);
		drawExplosion(3, getPxlForCase(0,row,col) );  
		break;

	case ReDraw:
		Redraw();
		break;

	case GlobalReset:
		Redraw();
		break;

	case PlayerWon:
		sayStatusBar(MSG_WIN);
		Redraw();
		break;

	case PlayerLost:
		sayStatusBar(MSG_LOSE);
		Redraw();
		break;

	default: ;
	}

}

/////////////////////////////////////////////////////////////////////////////
//	User's pen handlers
/////
//
void PlayerWdw::onPenDown(const PegPoint &p)
{
	int row, col;
	if(playerDoc()->myTurn() && findCase(p,row,col)==1)
	{
		if(playerDoc()->getCaseType(1,row,col) == UnKnown)
		{
			m_choosedPoint = p;
			playerDoc()->shoot(row,col);
			sayStatusBar(MSG_WAITING_OPP);
		}
	}
	else if(playerDoc()->placingBoats() && findCase(p,row,col)==0)
	{
		m_boatBaseRow = row;
		m_boatBaseCol = col;
		m_boatBasePlaced = true;
		tryBoat(row,col);
		Redraw();
	}

}

/////
//
void PlayerWdw::onPenMove(const PegPoint &p)
{
	int row, col;
	if(playerDoc()->placingBoats() && m_boatBasePlaced && findCase(p,row,col)==0)
	{
		tryBoat(row,col);
		Redraw();
	}
}

/////
//
void PlayerWdw::onPenUp(const PegPoint &p) 
{
	int row, col;
	if(playerDoc()->placingBoats() && m_boatBasePlaced && findCase(p,row,col)==0)
	{
		if(m_boatValid) 
			playerDoc()->placeBoat(m_boatTop, m_boatLeft, m_boatHoriz, m_boatSize);
		m_boatBasePlaced = false;
		Redraw();
	}
}

/////////////////////////////////////////////////////////////////////////////

void PlayerWdw::tryBoat(int row, int col)
{
	if(row==m_boatBaseRow && col==m_boatBaseCol)
	{
		m_boatValid = false;
		m_boatLeft = col;
		m_boatTop = row;
		m_boatSize = 1;
		return;
	}

	int d1 = row-m_boatBaseRow; if(d1<0) d1=-d1;
	int d2 = col-m_boatBaseCol; if(d2<0) d2=-d2;
	m_boatHoriz = (d1<=d2);
	if(m_boatHoriz)
	{
		m_boatTop = m_boatBaseRow;
        m_boatLeft = (col<m_boatBaseCol?col:m_boatBaseCol);
	}
	else
	{
		m_boatTop = (row<m_boatBaseRow?row:m_boatBaseRow);
        m_boatLeft = m_boatBaseCol;
	}

	m_boatSize = (m_boatHoriz? d2 : d1)+1;

	m_boatValid = playerDoc()->canPlaceBoat(m_boatTop,m_boatLeft,m_boatHoriz,m_boatSize);
}

/////////////////////////////////////////////////////////////////////////////

int PlayerWdw::findCase(const PegPoint &p, int &row, int &col)
{
	int width = SIZE_GRID*(SIZE_CASE+1);
	if( p.y < ME_GRID_POS_Y + width 
		&& p.y > ME_GRID_POS_Y 
		&& p.x < ME_GRID_POS_X + width 
		&& p.x > ME_GRID_POS_X	) 
	{
		row = (p.y-(ME_GRID_POS_Y+1))/(SIZE_CASE+1), 
		col = (p.x-(ME_GRID_POS_X+1))/(SIZE_CASE+1);
		return 0;
	}
	else if( p.y < OPP_GRID_POS_Y + width 
		&& p.y > OPP_GRID_POS_Y 
		&& p.x < OPP_GRID_POS_X + width 
		&& p.x > OPP_GRID_POS_X	)
	{
		row = (p.y-(OPP_GRID_POS_Y+1))/(SIZE_CASE+1), 
		col = (p.x-(OPP_GRID_POS_X+1))/(SIZE_CASE+1);
		return 1;
	}

	return -1;		
}

/////////////////////////////////////////////////////////////////////////////







