/////////////////////////////////////////////////////////////////////////////
//			Custom Window System - MyMainFrame
//	
//				Julien Patte (orwell01@hotmail.com)
//	
//					version 0.9 (incomplete) - 29 jul 2005
//
// Note: You may freely use these files, but it's at your own risks ;)
/////////////////////////////////////////////////////////////////////////////

#include "../MainFrame/MyMainFrame.h"

PegRect DummyFrame::m_rect = {0,0,0,0};

SIGNED DummyFrame::Message(const PegMessage &msg)
{
	return MyMainFrame::mf()->Message(msg);
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

#define DEF_MENU_QUIT		3851
#define MYMF_SEND_SIGNAL	3852

PegRect MyMainFrame::m_bigRect = { 0,0,159,231 };
PegRect MyMainFrame::m_smallRect = { 0,0,159,130 };

MyMainFrame* MyMainFrame::m_instance = NULL;

/////////////////////////////////////////////////////////////////////////////

MyMainFrame::MyMainFrame(PegRect rect, WORD style)
	: PegWindow(rect,style)
	, m_dummy()
{
	m_titleBar = NULL;
	m_menuBar = NULL;
	m_toolBar = NULL;
	m_statusBar = NULL;

	m_instance = this;
	m_activeWin = NULL;

	m_keyPadOn = false;
	m_fullScreen = false;
	m_allowResizeKey = true;

	m_client = new PegWindow(mClient,FF_NONE);
	m_client->RemoveStatus(PSF_ACCEPTS_FOCUS);
	//m_client->SetScrollMode(WSM_AUTOSCROLL);
	Add(m_client);

	//getTitle();
	//getMenuBar();
	//getStatusBar();
}

/////////////////////////////////////////////////////////////////////////////

MyMainFrame::~MyMainFrame(void)
{
	Destroy(m_client);

	if(m_titleBar) destroyTitle();
	if(m_menuBar) destroyMenuBar();
	if(m_toolBar) destroyToolBar();
		
	if(m_statusBar) 
	{
		Presentation()->Destroy(m_statusBar);
		m_statusBar = NULL;
	}

	if(keypadOn()) showKeypad(false);
}

/////////////////////////////////////////////////////////////////////////////

MyMainFrame* MyMainFrame::mf()
{
	if(m_instance==NULL) m_instance = new MyMainFrame();
	return m_instance;
}

/////////////////////////////////////////////////////////////////////////////

void MyMainFrame::setTitle(PegTitle* title) 
{
	if(m_titleBar) Destroy(m_titleBar);
	m_titleBar = title;
	Add(m_titleBar);
	resizeClient();
}

/////////////////////////////////////////////////////////////////////////////

void MyMainFrame::destroyTitle() 
{
	if(m_titleBar) 
	{
		Destroy(m_titleBar);
		m_titleBar = NULL;
		resizeClient();
	}
}

/////////////////////////////////////////////////////////////////////////////

void MyMainFrame::setMenuBar(PegMenuBar* menu)
{
	if(m_menuBar) Destroy(m_menuBar);
	m_menuBar = menu;
	Add(m_menuBar);
	resizeClient();
}

/////////////////////////////////////////////////////////////////////////////

void MyMainFrame::destroyMenuBar()
{
	//if(m_menuBar) 
	//{
	//	Destroy(m_menuBar);
	//	m_menuBar = NULL;
	//	resizeClient();
	//}
}

/////////////////////////////////////////////////////////////////////////////

void MyMainFrame::setToolBar(PegToolBar* tool)
{
	if(m_toolBar) Destroy(m_toolBar);
	m_toolBar = tool;
	Add(m_toolBar);
	resizeClient();
}

/////////////////////////////////////////////////////////////////////////////

void MyMainFrame::destroyToolBar()
{
	if(m_toolBar) Destroy(m_toolBar);
	m_toolBar = NULL;
	resizeClient();
}

/////////////////////////////////////////////////////////////////////////////

PegTitle* MyMainFrame::getTitle()
{
	if(m_titleBar==NULL) 
	{
		PegTitle* t = new PegTitle("");
		t->Resize(defaultTitleRect());
		setTitle(t);
	}
	return m_titleBar;
}

/////////////////////////////////////////////////////////////////////////////

PegMenuBar* MyMainFrame::getMenuBar()
{
	if(m_menuBar==NULL) 
	{
		PegMenuDescription defaultMenu[] = 
			{ {"Quitter", DEF_MENU_QUIT, AF_ENABLED|TT_COPY, NULL}, { "", 0,0,NULL } };
		PegMenuBar* t = new PegMenuBar(defaultMenu);
		t->Resize(defaultMenuBarRect());
		setMenuBar(t);
	}
	return m_menuBar;
}

/////////////////////////////////////////////////////////////////////////////

PegToolBar* MyMainFrame::getToolBar()
{
	if(m_toolBar==NULL)
	{
		//PegToolBar* t = new PegToolBar();
		//t->Resize(defaultToolBarRect());
		//setToolBar(t,this);
	}
	return m_toolBar;
}

/////////////////////////////////////////////////////////////////////////////

PegStatusBar* MyMainFrame::getStatusBar()
{
	if(m_statusBar==NULL) 
	{
		m_statusBar = new PegStatusBar();
		m_statusBar->RemoveStatus(PSF_ACCEPTS_FOCUS);
		m_statusBar->AddTextField(1, 1);
		Presentation()->AddToEnd(m_statusBar);
	}
	return m_statusBar;
}

/////////////////////////////////////////////////////////////////////////////

void MyMainFrame::statusBarText(PEGCHAR* string)
{
	getStatusBar()->SetTextField(1,string);
}

/////////////////////////////////////////////////////////////////////////////

void MyMainFrame::addWindow(MyWindow* win)
{
	if(win==NULL) return;
	if(win->posCorrected()==false) placeInClient(win, win->mReal.wLeft, win->mReal.wTop, false);
	m_client->Add(win);
	if(m_activeWin) m_otherWin.Insert(m_activeWin);
	m_activeWin = win;
	Presentation()->MoveFocusTree(win);
		
	PegMessageWindow* pWin = new PegMessageWindow("",""); 
	delete pWin; 
	// Yes, I know... this is stupid.
	// But don't ask me why the system can freeze if you don't write it...
}

/////////////////////////////////////////////////////////////////////////////

void MyMainFrame::removeWindow(MyWindow* win, bool destroy)
{
	if(win==NULL) return;
	if(win==m_activeWin)
	{
		if(m_otherWin.Count())
		{
			m_activeWin = (MyWindow*) m_otherWin.Get();
			m_otherWin.Remove(m_activeWin);
			Presentation()->MoveFocusTree(m_activeWin);
		}
		else m_activeWin = NULL;
	}
	else if(m_otherWin.Contains(win)) m_otherWin.Remove(win);

	if(destroy) m_client->Destroy(win);
	else m_client->Remove(win);
}

/////////////////////////////////////////////////////////////////////////////

bool MyMainFrame::containsWindow(MyWindow* win)
{
	return (win==m_activeWin || m_otherWin.Contains(win));
}

/////////////////////////////////////////////////////////////////////////////

void MyMainFrame::setActiveWindow(MyWindow* win)
{
	if(containsWindow(win)==false) addWindow(win);
	else
	{
		m_activeWin = win;
		Presentation()->MoveFocusTree(win);
	}
}

/////////////////////////////////////////////////////////////////////////////

void MyMainFrame::placeInClient(MyWindow* win, int x, int y, bool redrawClient)
{
	PegRect r = win->mReal;
	r.MoveTo(m_client->mClient.wLeft + x, m_client->mClient.wTop + y);
	win->Resize(r);
	win->setPosCorrected(true);
	if(redrawClient) m_client->Redraw();
}

/////////////////////////////////////////////////////////////////////////////

void MyMainFrame::resizeClient()
{
	m_client->Resize(clientRect());
}

/////////////////////////////////////////////////////////////////////////////
//#include <pmessage.hpp>
SIGNED MyMainFrame::Message(const PegMessage &msg)
{
    bool ok;
	switch(msg.wType)
    {
        //case PM_ADD:   
        //	addWindow( (PegWindow*)msg.pSource );
        //	break;

		case PM_DRAW:
			Invalidate();
			Draw();
			break;

        case HM_SYS_KEYBOARD:
            toggleKeypad();
            break;

		//case HM_SYS_RESUME:
		//	break;

		case HM_SYS_ZOOM:
			if(m_allowResizeKey) setFullScreenApp(!m_fullScreen);
			break;

        case HM_SYS_CLEAR:
			if(m_activeWin && m_activeWin->onClearKeyPressed()) 
				removeWindow(m_activeWin);
            break;

		case PM_FIRST_START:
			if(m_activeWin) Presentation()->MoveFocusTree(m_activeWin);
			break;

 		case SIGNAL(DEF_MENU_QUIT, PSF_CLICKED):
			programQuit();
			break;

		case MYMF_SEND_SIGNAL:
			((MyWindow*)msg.pSource)->onSignal(msg.iData);
			break;
		
       default:
			ok = false;
			if(m_activeWin)	m_otherWin.Insert(m_activeWin);
			ent ptr2;
			CPListIterator it2(&m_otherWin);	
			for(ptr2 = it2.Head(); ptr2!=NULL; ptr2 = it2.Next())
			{
				ent ptr;
				CPListIterator it( ((MyWindow*)ptr2)->handledSignalsList() );
				for(ptr = it.Head(); ptr!=NULL; ptr = it.Next())
				{
					if( msg.wType == *((WORD*)ptr) )
					{
						prepareSignal((MyWindow*)ptr2, msg.wType);
						ok=true;
						break;
					}
				}
			}
		
			m_otherWin.Remove(m_activeWin);

			if(!ok) return PegWindow::Message(msg);
    }
    return 0;
}

/////////////////////////////////////////////////////////////////////////////

void MyMainFrame::prepareSignal(MyWindow* win, WORD signal)
{
	PegMessage msg(this,MYMF_SEND_SIGNAL);
	msg.iData = signal;
	msg.pSource = win;
	MessageQueue()->Push(msg);
}

/////////////////////////////////////////////////////////////////////////////

void MyMainFrame::Draw()
{
    BeginDraw();
	DrawFrame();

	DrawChildren();
    EndDraw();
}

/////////////////////////////////////////////////////////////////////////////

void MyMainFrame::programQuit()
{

}

/////////////////////////////////////////////////////////////////////////////

PegRect MyMainFrame::defaultTitleRect()
{
	PegRect r = { 0,0,159,13 };
	return r;
}

/////////////////////////////////////////////////////////////////////////////

PegRect MyMainFrame::defaultMenuBarRect()
{
	PegRect r = { 0,0,159,13 };
	if(m_titleBar) { r.MoveTo(0,m_titleBar->mReal.wBottom+1); }
	return r;
}

/////////////////////////////////////////////////////////////////////////////

PegRect MyMainFrame::defaultToolBarRect()
{
	PegRect r = { 0,0,159,16 };
	if(m_titleBar) { r.MoveTo(0,m_titleBar->mReal.wBottom+1); }
	if(m_menuBar) { r.MoveTo(0,m_menuBar->mReal.wBottom+1); }
	return r;
}

/////////////////////////////////////////////////////////////////////////////

PegRect MyMainFrame::clientRect()
{
	PegRect r = mClient;
	if(m_fullScreen==false)
	{
		if(m_toolBar)		r.wTop = m_toolBar->mReal.wBottom + 3;
		else if(m_menuBar)	r.wTop = m_menuBar->mReal.wBottom + 3;
		else if(m_titleBar) r.wTop = m_titleBar->mReal.wBottom + 3;
	}
	return r;
}

/////////////////////////////////////////////////////////////////////////////

void MyMainFrame::toggleKeypad()
{
	showKeypad( ! keypadOn() );
}

/////////////////////////////////////////////////////////////////////////////

void MyMainFrame::showKeypad(bool on)
{
	if( ! (on ^ keypadOn()) ) return;

	m_keyPadOn = on;

	if(on)
	{
		Presentation()->AddToEnd(getKeypad());
		Resize(m_smallRect);
		resizeClient();
	    Presentation()->Draw();
	}
	else
	{
		Presentation()->Remove(getKeypad());
		Resize(m_bigRect);
		resizeClient();
	    Presentation()->Draw();
	}
}

/////////////////////////////////////////////////////////////////////////////

bool MyMainFrame::keypadOn()
{
	return m_keyPadOn;
}

/////////////////////////////////////////////////////////////////////////////

void MyMainFrame::setFullScreenApp(bool on)
{
	m_fullScreen = on;

	if(keypadOn()) showKeypad(false);

	resizeClient();
	Redraw();
}

/////////////////////////////////////////////////////////////////////////////




