/////////////////////////////////////////////////////////////////////////////
//		Link Communication System, with USB support
//	
//		Julien Patte (orwell01@hotmail.com)
//			based on deadbird's studies (deadbird666@aol.com) :)
//	
//		version 1.0 - 29 jul 2005
//
// Note: You may freely use these files, but it's at your own risks ;)
// Warning: Serial communication between ClassPad and computer
//	does not seem to work :(
/////////////////////////////////////////////////////////////////////////////


#include "../USBComm/USBComm.h"

byte LinkPortType = IM_LINK_USB; // IM_LINK_SRL3PIN

#ifdef WIN32

#include <stdlib.h>
#include <Windows.h>

#define CHOICE(a,b) ( LinkPortType==IM_LINK_USB ? a : b )

void linkInit() 
{ 
	UsbLinkInit();
	LinkInitial();
}

word linkOpen(word type) 
{ 
	LinkPortType = type&0x00F0;
	return CHOICE(UsbOpenComm(), LinkCommOpen(type)); 
}

word linkClose(word type) 
	{ return CHOICE(UsbCloseComm(), LinkClose(type)); }

word linkSend(byte data) 
	{ return CHOICE(UsbSend(&data,1), LinkTxByte(data)); }

word linkSend(byte* data, word size) 
	{ return CHOICE(UsbSend(data,size), LinkWrite(data,size, LinkPortType)); }

word linkReceive(byte &data) 
	{ return CHOICE(UsbReceive(&data,1), LinkRxByte(&data)); }

word linkReceive(byte* data, word size) 
	{ return CHOICE(UsbReceive(data,size), LinkRead(data,size)); }

word linkClearRecBuff() 
	{ return CHOICE(0, LinkClearRxBuf()); }

word linkClearTrBuff()
	{ return CHOICE(0, LinkClearTxBuf()); }


static int (_stdcall *UsbEnumUSB)(DWORD, char*, DWORD) = NULL;
static HANDLE UsbHandle;

/////////////////////////////////////////////////////////////////////////////

void UsbLinkInit()
{
	HMODULE cpdll = LoadLibrary("EnumDev.dll");
	
	if (cpdll==0) exit(0);
		//get a pointer to _PVEnumUsbA@12
	UsbEnumUSB = (int (_stdcall *)(DWORD, char*, DWORD))GetProcAddress(cpdll, "_PVEnumUsbA@12");
	BOOL ret = ResetEvent(cpdll); //maybe not needed
}

/////////////////////////////////////////////////////////////////////////////

word UsbOpenComm()
{
	char device[1024];
	device[0]='\0';
	//get the CP path!
	//WARNING : CP must have called successfuly LinkInitial for this not to fail
	for(int i=0; device[0]==0 && i<MAX_CONNECTION_TRIES; i++)
	{
		UsbEnumUSB(0, device, sizeof(device));
		if(device[0]==0) Sleep(TIME_BETWEEN_TRIES);
	}

	if(device[0]==0) return IM_COMM_TIME_OUT_ERR;

    //create the channel itself
	UsbHandle = CreateFile(device, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	return IM_COMM_NO_ERR;
}

/////////////////////////////////////////////////////////////////////////////

word UsbCloseComm()
{
	return (CloseHandle(UsbHandle) ? IM_COMM_NO_ERR : IM_COMM_OTHER_ERR);
}

/////////////////////////////////////////////////////////////////////////////

word UsbSend(byte* data, word size)
{
	DWORD ok;
	WriteFile(UsbHandle, data, size, &ok, 0);
	return (ok==size ? IM_COMM_NO_ERR : IM_COMM_OTHER_ERR);
}

/////////////////////////////////////////////////////////////////////////////

word UsbReceive(byte* data, word size)
{
	DWORD ok;
	ReadFile(UsbHandle, data, size, &ok, 0);
	return (ok==size ? IM_COMM_NO_ERR : IM_COMM_OTHER_ERR);
}

/////////////////////////////////////////////////////////////////////////////

#endif
