/////////////////////////////////////////////////////////////////////////////
//		Link Communication System, with USB support
//	
//		Julien Patte (orwell01@hotmail.com)
//			based on deadbird's studies (deadbird666@aol.com) :)
//	
//		version 1.0 - 29 jul 2005
//
// Note: You may freely use these files, but it's at your own risks ;)
// Warning: Serial communication between ClassPad and computer
//	does not seem to work :(
/////////////////////////////////////////////////////////////////////////////

typedef unsigned short word;
typedef unsigned char byte;
typedef unsigned long dword;

extern "C" {
	void LinkInitial(void);
	word LinkClose(word);
	word LinkCommOpen(word);
	word LinkRxByte(byte*);
	word LinkTxByte(byte);
	word LinkRead(byte*, word);
	word LinkWrite(byte*, word, word);
	word LinkClearRxBuf();
	word LinkClearTxBuf();
}

extern byte LinkPortType;

#include <ClassPad.h>

/////////////////////////////////////////////////////////////////////////////

#ifdef WIN32

#define MAX_CONNECTION_TRIES	10
#define TIME_BETWEEN_TRIES		500

// Functions that use USB or Serial communications, depending on the
// specified port when calling linkOpen()
void linkInit();
word linkOpen(word type);
word linkClose(word type);
word linkSend(byte data);
word linkSend(byte* data, word size=1);
word linkReceive(byte &data);
word linkReceive(byte* data, word size=1);
word linkClearRecBuff();
word linkClearTrBuff();

// Functions for USB communication only (only available on Windows system)
void UsbLinkInit();
word UsbOpenComm();
word UsbCloseComm();
word UsbSend(byte* data, word size);
word UsbReceive(byte* data, word size);

/////////////////////////////////////////////////////////////////////////////

#else

// Pending functions, when compiling for ClassPad
inline void linkInit() 
	{ LinkInitial(); }

inline word linkOpen(word portAndSpeed) 
	{ LinkPortType = (portAndSpeed&0x00F0); return LinkCommOpen(portAndSpeed); }

inline word linkClose(word type) 
	{ return LinkClose(type); }

inline word linkSend(byte data) 
	{ return LinkTxByte(data); }

inline word linkSend(byte* data, word size) 
	{ return LinkWrite(data,size, LinkPortType); }

inline word linkReceive(byte &data) 
	{ return LinkRxByte(&data); }

inline word linkReceive(byte* data, word size) 
	{ return LinkRead(data,size); }

inline word linkClearRecBuff() 
	{ return LinkClearRxBuf(); }

inline word linkClearTrBuff() 
	{ return LinkClearTxBuf(); }

#endif

/////////////////////////////////////////////////////////////////////////////

